; Title - Bitmap Display
; By    - Adam Trionfo
;
;         1.3  - December 6, 2016
;              - Fixed to work with recent versions of MESS/MAME Emulator.
;                Changed Interrupt Mode from $18 to $08 (screen
;                interrupts only).  Must press Reset to begin program properly.
;         1.21 - December 6, 2003
;         1.2  - March 25, 2002
; 
; This program could not have been written without using
; Mike White's "Astro BASIC 4x2 Multicart Loader" and
; and the "Nutting Manual" as a reference.
;
;  
; About this Program
; ------------------ 
;
; This program displays a 4000 byte Astrocade bit-map
; screen
;
; Assembling this Program
; -----------------------
;    This file will assemble with ZMAC 1.3 (a little
; known, freely distributable Z-80 assembler (with C
; source), that has a 25-year history.  ZMAC can be
; compiled under just about any O.S. in existence, so
; try it out. 
;  
; To assemble Z-80 source code using ZMAC:
;  
; zmac -i -m -o <outfile> -x <listfile> <filename>
; 
; For example, to assemble this Astrocade Z-80 program:
;    
; zmac -i -m -o titlebal.bin -x titlebal.lst titlebal.asm

INCLUDE "HVGLIB.H"

        ORG    $2000    ; First byte of Cartridge
        DB     $55      ; Bally system Sentinel

        B1VEC  EQU 4F00H		;VECTOR BLOCK FOR BALL TEST

        DW     MENUST   ; Next menu link
        DW     PRGNAM   ; Address of title for program
        DW     PRGST    ; Jump here if prog is selected


BITMAP:
; 4000 byte bitmap graphic
INCLUDE "TITLEMUL.ASM"

;       ORG    $2FA7

PRGST:  DI
        SYSTEM (INTPC)

        DO     (SETOUT)
        DB     $B0        ; Vertical Blanking Line
        DB     $2C        ; Left/Right Color Boundary
        DB     00001000B  ; Set Bit 3 of INterrupt MODe (Screen Interrupts Only)
 
        DO     (COLSET)
        DW     COLTAB     ; Color Table

        DO     (MOVE)
        DW     NORMEM     ; Destination
        DW     4000D      ; Bytes to move
        DW     BITMAP     ; Source Address
        EXIT

	LD	IX,B1VEC	;BEGIN SETUP OF VECTOR BLOCK
	LD	(IX+VBMR),8	;MAGIC: EXPAND
	LD	(IX+VBSTAT),192	;STATUS: ACTIVE+BLANKING
	LD	(IX+VBTIMB),1	;TIMES: 1 (NUMBER OF TIMES DELTA IS ADDED TO POSITION)
	LD	(IX+VBDXL),0	;DELTA (DIRECTION)X LOW ORDER BYTE (FRACTIONS)
	LD	(IX+VBDXH),1	;1 PIXEL PER CALL MOTION (RIGHT)
	LD	(IX+VBXL),0	;INITIAL POSITION LOW ORDER BYTE (FRACTIONS)
	LD	(IX+VBXH),0	;ACTUAL X POSITION ON SCREEN
	LD	(IX+VBXCHK),3	;X LIMIT CHECKING: ON + REVERSING (BOUNCE)
	LD	(IX+VBDYL),0	;DELTA (DIRECTION)Y LOW ORDER BYTE (FRACTION)
	LD	(IX+VBDYH),1	;1 PIXEL PER CALL MOTION (DOWN)
	LD	(IX+VBYL),0	;INITIAL POSSITION LOW ORDER BYTE (FRACTION)
	LD	(IX+VBYH),0	;ACTUAL Y POSITION ON SCREEN
	LD	(IX+VBYCHK),3	;Y LIMIT CHECKING: ON + REVERSING (BOUNCE)
	LD	(IX+VBOAL),0	;OLD ADDRESS: CLEARED FOR NOW
	LD	(IX+VBOAH),0	;END VECTOR BLOCK SETUP

	LD	A,0CH		;COLOURS TO EXPAND TO
	OUT	(XPAND),A	;INTO EXPANSION PORT

LOOP	SYSSUK	VWRITR		;DRAW BALL
	DEFW	B1VEC		;VECTOR BLOCK TO USE
	DEFW	BALL1		;IMAGE TO DRAW

	LD	IX,B1VEC
	LD	(IX+VBOAL),E	;COPY SCREEN ADDRESS TO VECTORBLOCK?
	LD	(IX+VBOAH),D	;

	SYSSUK	PAWS		;PAUSE FOR
	DEFB	1		;ONE CYCLE

	SYSSUK	VBLANK		;ERASE BALL
	DEFW	B1VEC		;VECTOR BLOCK
	DEFB	2		;X SIZE IN BYTES
	DEFB	6		;Y SIZE

	SYSSUK	VECT		;MOVE BALL
	DEFW	B1VEC		;VECTOR BLOCK TO USE
	DEFW	BLMT		;LIMITS TABLE TO USE

	LD	IX,B1VEC
	LD	(IX+VBTIMB),1	;RE ACTIVATE?
	JR	LOOP		;BACK TO DRAW BALL

;********************
;* BALL IMAGE BLOCK *
;********************

BALL1	DEFB	0	;NO X OFFSET
	DEFB	0	;NO Y OFFSET
	DEFB	1	;1 BYTE WIDE
	DEFB	6	;6 BYTES TALL
	DEFB	01111000B	
	DEFB	11011100B
	DEFB	10111100B	
	DEFB	10111100B
	DEFB	11111100B
	DEFB	01111000B

;***************
;* BALL LIMITS *
;***************

BLMT	DEFB	0	;X LOWER LIMIT (LEFT EDGE)
	DEFB	153	;X UPPER LIMIT (RIGHT EDGE)
			;-8 FOR 8 PIXEL WIDE BALL
	DEFB	0	;Y LOWER LIMIT (TOP EDGE)
	DEFB	89	;Y UPPER LIMIT (BOTTOM EDGE)
			;-6 FOR 6 PIXEL HIGH BALL

;******************
;* Color Table #1 *
;******************

        ORG    $3FE7
COLTAB: DB     $00        ; Color 3 Left  - Black
        DB     $5A        ; Color 2 Left  - Red
        DB     $77        ; Color 1 Left  - Yellow
        DB     $72        ; Color 0 Left  - Brown
        DB     $00        ; Color 3 Right - Black
        DB     $5A        ; Color 2 Right - Red
        DB     $77        ; Color 1 Right - Yellow
        DB     $72        ; Color 0 Right - Brown

        ORG    $3FEF      ; Program Name
PRGNAM: DB     'D','I','S','P','L','A','Y',' '
        DB     'B','I','T','M','A','P'
        DB     $00        ; End

        ORG    $3FFF
        DB     $00        ; End - Last byte
